@@ -39,6 +39,6 @@ if (!isGeneric("postProb")) {
 }
 
 if (!isGeneric("rnlp")) {
-  setGeneric("rnlp", function(y, x, msfit, priorCoef, priorVar, niter=10^3, burnin=round(niter/10), thinning=1) standardGeneric("rnlp"))
+  setGeneric("rnlp", function(y, x, m, V, msfit, priorCoef, priorVar, niter=10^3, burnin=round(niter/10), thinning=1) standardGeneric("rnlp"))
 }
 
